from .dbserver.data_manager import data_manager

class Data(object):
    Base = None

    @staticmethod
    def init(db_config):
        Data.Base = data_manager(db_config)

    @staticmethod
    def check_connections():
        sql_pool = Data.Base.sql_pool
        busy_list = []
        all_list = []
        free_list = []
        sql_executing = {}
        for sql_id in sql_pool:
            sql = sql_pool[sql_id]
            if sql.is_busy() == True:
                busy_list.append(sql_id)
                sql_executing[sql_id] = sql.executing_query

            if sql.is_busy() == False:
                free_list.append(sql_id)
            all_list.append(sql_id)

        result = {
            'busy_base': {
                'count': len(busy_list),
                'lists': busy_list,
                'executing': sql_executing,
            },
            'free_base': {
                'count': len(free_list),
                'lists': free_list
            },
            'all_base': {
                'count': len(all_list),
                'lists': all_list
            }
        }
        return result

    @staticmethod
    def add_new_sql():
        return Data.Base.new()

    @staticmethod
    def create(table, colums, show_sql=False):
        return Data.Base.create(table, colums, show_sql)

    @staticmethod
    def insert(table, params, is_commit=True, show_sql=False):
        data = Data.Base.insert(table, params, is_commit, show_sql)
        return data

    @staticmethod
    def find(table, conditions, fields=('*',), order=None, show_sql=False):
        return Data.Base.find(table, conditions, fields, order, show_sql)

    @staticmethod
    def select(table, conditions, fields=('*',), group=None,order=None, limit=None,show_sql=False):
        return Data.Base.select(table, conditions, fields,group, order, limit,show_sql)

    @staticmethod
    def update(table, conditions, params, is_commit=True, show_sql=False):
        Data.Base.update(table, conditions, params, is_commit, show_sql)
        return

    @staticmethod
    def delete(table, conditions, is_commit=True, show_sql=False):
        data = Data.Base.delete(table, conditions, is_commit, show_sql)
        return data

    @staticmethod
    def find_last(table, conditions, info, limit, fields="*", show_sql=False):
        return Data.Base.find_last(table, conditions, info, limit, fields, show_sql)

    @staticmethod
    def query(sql, show_sql=False):
        return Data.Base.query(sql, show_sql)




if __name__ == '__main__':
    res = Data.select('withdraw_record',[('user_id','in',tuple(range(1,10)))],group=['user_id'],fields=['user_id','sum(share)'],show_sql=False)
    for i in res:
        print(i)
        print(i['sum(share)'])


import threading
import time
import heapq
from threading import Timer
import random
# import _thread as thread

TIME_ACCURACY = 1  # 时间精度，时间精度不是越小越好！你的Task每次循环超过了这个值,将影响准确度


def IntervalTask(sec, func, args=(), immediatly=True, thread_name=''):
    def run(*args):
        if immediatly is False:
            time.sleep(sec)
        while 1:
            func(*args)
            time.sleep(sec)

    t = threading.Thread(target=run, args=args)
    if thread_name != '':
        t.name = thread_name
    t.start()
    # t.join()


def normal_task(sec, func, args=(), thread_name=''):
    def run(*args):
        time.sleep(sec)
        func(*args)

    t = threading.Thread(target=run, args=args)
    if thread_name != '':
        t.name = thread_name
    t.start()
    # t.join()
    # threading.enumerate()
    # print(threading.enumerate())

# def interval_pro(sec,func,args=()):


class Scheduler(object):
    tasks = []

    @staticmethod
    def add(task):
        heapq.heappush(Scheduler.tasks, (task.get_runtime(), task))

    @staticmethod
    def run(is_timer=False):
        now_time = time.time()
        while Scheduler.tasks and now_time >= Scheduler.tasks[0][0]:
            _, task = heapq.heappop(Scheduler.tasks)
            task.call()

            if task.is_cycle():
                task.up_runtime()
                Scheduler.add(task)

        if is_timer is True:
            t = Timer(TIME_ACCURACY, Scheduler.run, args=[True])
            t.start()


'''
定时任务
func:要定时执行的方法
args:函数的参数（tuple）
执行start()此定时器生效
'''


class Task(object):
    def __init__(self, func, args=()):
        self._func = func
        self._args = args
        self._runtime = time.time()
        self._interval = 0
        self._cycle = False

    def __lt__(self, other):
        return False

    def call(self):
        self._func(*self._args)

    def up_runtime(self):
        self._runtime += self._interval

    def is_cycle(self):
        return self._cycle

    def get_runtime(self):
        return self._runtime + random.random() / 100

    def start(self):
        Scheduler.add(self)


'''
作用：简单定时任务
runtime:开始时间(时间戳)
'''

# class NormalTask(Task):
#
#     def __init__(self, runtime, func, args=()):
#         Task.__init__(self, func, args)
#         self._runtime = runtime
#         self.start()


'''
作用：倒计时定时任务
countdown:倒计时，这个时间后开始执行（单位：秒）
'''


class CountdownTask(Task):

    def __init__(self, countdown, func, args=()):
        Task.__init__(self, func, args)
        self._runtime += countdown
        self.start()


'''
作用：循环定时任务
interval:循环周期（单位：秒）
'''

# 使用方法：
'''
res = []
for i in data:
    r = func(data)
    res.append(data)
return res
=======顺序执行======

res = super_charger(func,data)
#####增压器#######
=======加速执行======
# '''


class func_engine(threading.Thread):
    def __init__(self, func, args=()):
        super(func_engine, self).__init__()
        self.func = func
        self.args = args

    def run(self):
        self.result = self.func(*self.args)

    def get_result(self):
        try:
            return self.result
        except Exception:
            return None
    # 并发优化


def turbo(func, arg_list):
    # global func_runner
    # 函数增压器！
    li = []
    for i in arg_list:
        runner = func_engine(func, args=i)
        li.append(runner)
        runner.start()

    result = []
    for i in li:
        i.join()
        result.append(i.get_result())
    return result


# ↓最终加速工具，请调用这个函数
def super_charger(func, arg_list):
    return turbo(func, arg_list)


# ↑最终加速工具，请调用这个函数


if "__main__" == __name__:
    # res = [(1,2,3),(4,5,6),(7,8,9)]
    # print(turbo(foo,res))
    pass



import codecs
import os

from setuptools import setup, find_packages

def read(fname):
    """
    定义一个read方法，用来读取目录下的长描述
    我们一般是将README文件中的内容读取出来作为长描述，这个会在PyPI中你这个包的页面上展现出来，
    你也可以不用这个方法，自己手动写内容即可，
    PyPI上支持.rst格式的文件。暂不支持.md格式的文件，<BR>.rst文件PyPI会自动把它转为HTML形式显示在你包的信息页面上。
    """
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    # 以下为必需参数
    name='anduindata',  # 模块名
    version='1.0.0',  # 当前版本
    description='a mysql connector',  # 简短描述
    py_modules=["anduindata"], # 单文件模块写法
    # ckages=find_packages(exclude=['contrib', 'docs', 'tests']),  # 多文件模块写法
    license='MIT',
    long_description = read("README.txt"),
    author='campanula',
    author_email='421248329@qq.com',
    platforms = 'any',
    keywords = "mysql",
    # packages = find_packages('anduin'),
    # package_dir = {'anduin':'*'},
    classifiers = [
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python',
            'Intended Audience :: Developers',
            'Operating System :: OS Independent',
        ],
    url = 'https://github.com/campanulamediuml/Anduin',
    install_requires=[
          'pymysql==0.10.1',
      ],
    include_package_data=True,
    zip_safe=True,
    packages = ['anduin','anduin/dbserver'],
    python_requires='>=3.2',

)


import time
from ..Scheduler import IntervalTask
from ..dbserver.base import Base
# from config.config import db_config


class data_manager(object):
    min_keep_connection = 10

    def __init__(self,db_config):
        self.t_data = db_config
        self.sql_pool = {}
        self.new()
        print('创建首批数据库链接实例')
        IntervalTask(10,self.keep_connect)
    # def kill_hanged_connection(self):
    #     dead_sql = []
    #
    def keep_connect(self):
        can_update_sql = []
        for sql in self.sql_pool.values():
            if int(time.time())-sql.get_last_connect_time()  > 30:
                can_update_sql.append(sql)

        count = 0
        kill_connect = []
        for sql in can_update_sql:
            if count > data_manager.min_keep_connection:
                kill_connect.append(id(sql))
                continue
            if sql.is_busy() == False:
                sql.become_busy()
                sql.keep_connect()
                # print(sql.id,'进行心跳')
                sql.become_free()
                count += 1
            # if count > data_manager.min_keep_connection:
            #
            #     break
        self.kill_unused_connection(kill_connect)

        return
    def kill_unused_connection(self,kill_connect):
        count = 0
        for sql_id in kill_connect:
            if sql_id in self.sql_pool:
                if self.sql_pool[sql_id].is_busy() is False:
                    self.sql_pool.pop(sql_id)
                    count += 1
        # print('计划清理',len(kill_connect),'个空闲连接，实际清理',count,'个')
        return


    def find_free_sql(self):
        for sql in self.sql_pool.values():
            if sql.is_busy() == False:
                sql.become_busy()
                return sql
        # print('数据库连接池全忙状态，创建新的数据库链接')
        sql = self.create_new_sql()
        sql.become_busy()
        self.add_new_sql(sql)
        # print('创建完毕')
        # dbg_db('数据库连接池全忙状态，创建新的数据库链接', '新连接id:', id(sql))
        return sql

    def add_new_sql(self,sql):
        self.sql_pool[id(sql)] = sql
        return

    def new(self):
        sql = self.create_new_sql()
        # print(id(sql))
        self.add_new_sql(sql)
        return


    def create_new_sql(self,):
        sql = Base(self.t_data['host'], self.t_data['user'], self.t_data['password'], self.t_data['database'])
        # sql.become_busy()
        # self.sql_pool.append(sql)
        return sql


    def get_tables(self):
        sql = self.find_free_sql()
        # sql.become_busy()
        print('执行这次sql请求的链接是', id(sql))
        result = sql._tables
        sql.become_free()
        return result


    def create(self,table, colums,show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        print('执行这次sql请求的链接是', id(sql))
        result = sql.create(table, colums,show_sql)
        sql.become_free()
        return result


    def insert(self,table, params, is_commit=True,show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.insert(table, params, is_commit,show_sql)
        sql.become_free()
        return result


    def find(self,table, conditions,fields=('*',), order=None,show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.find(table, conditions, fields, order,show_sql)
        sql.become_free()
        return result


    def select(self,table, conditions, fields=('*',), group=None,order=None,limit=None,show_sql=False):

        sql = self.find_free_sql()
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.select(table, conditions, fields, group,order,limit,show_sql)
        sql.become_free()
        return result


    def update(self,table, conditions, params, is_commit=True,show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.update(table, conditions, params, is_commit,show_sql)
        sql.become_free()
        return result


    def delete(self,table, conditions, is_commit=True,show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.delete(table, conditions, is_commit,show_sql)
        sql.become_free()
        return result


    def find_last(self,table, conditions, info, limit,fields="*",show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.find_last(table, conditions, info, limit,fields,show_sql)
        sql.become_free()
        return result

    def query(self,sql_query,show_sql=False):
        # print(sql)
        sql = self.find_free_sql()
        # sql.become_busy()
        print('执行这次sql请求的链接是', id(sql))
        result = sql.query(sql_query,show_sql)
        sql.become_free()
        return result

    # @staticmethod
    # def truncate(table):
    #     # print(sql)
    #     return Data.sql.truncate(table)
    def dump_table(self, table, sql):
        if sql.is_busy():
            return
        sql.become_busy()
        result = sql.select(table, [])
        sql.become_free()
        return result

if __name__=='__main__':
    data_manager(db_config)